
// Copyright (c) 2015, Tobias Mueller tm(at)tm3d.de
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the
//    distribution.
//  * All advertising materials mentioning features or use of this
//    software must display the following acknowledgement: This product
//    includes software developed by tm3d.de and its contributors.
//  * Neither the name of tm3d.de nor the names of its contributors may
//    be used to endorse or promote products derived from this software
//    without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "../common/I2C/USI_TWI_Master.h"
#include "../common/I2C/MAX1164x.h"
#include "../common/I2C/SHT2x.h"
#include <avr/pgmspace.h>

extern void OWINIT();
extern void EXTERN_SLEEP();


const int16_t hl[5] PROGMEM = {0,400,650,850,1000};  //H*10

//log values *1000
const int16_t il[5][5] PROGMEM= {
{470,	247,	140,	77,	49},
{336,	104,	0,  	-62,	-83},
{207,	-20,	-139,	-198,	-223},
{86,	-139,	-248,	-315,	-342},
{-30,	-248,	-371,	-431,	-462}};




extern void OWINIT();
extern void EXTERN_SLEEP();


uint8_t owid1[8]={0x28, 0xA3, 0xD9, 0x84, 0x00, 0x16, 0x05, 0x18};/**/
uint8_t owid2[8]={0x26, 0xA3, 0xD9, 0x84, 0x00, 0x16, 0x05, 0x67};/**/
uint8_t config_info1[16]={0x08,0x01, 0x00,0x00, 0x00,0x00, 0x00,0x00, 0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00};	
uint8_t config_info2[16]={0x01,0x06, 0x05,0x08, 0x04,0x07, 0x07,0x08, 0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00};	
	
#if (owid>128) 
#error "Variable not correct"
#endif

extern uint8_t mode;
extern uint8_t gcontrol;
extern uint8_t reset_indicator;
extern uint8_t alarmflag;
volatile uint8_t wdcounter=1;


#if  defined(__AVR_ATtiny24__)||defined(__AVR_ATtiny44__)  || defined(__AVR_ATtiny84__) || defined(__AVR_ATtiny24A__)||defined(__AVR_ATtiny44A__)  || defined(__AVR_ATtiny84A__)
ISR(WATCHDOG_vect) {
#else
ISR(WDT_vect) {
#endif 
	wdcounter++;
	if (reset_indicator==1) reset_indicator++;
	else if (reset_indicator==2) mode=0;


}


typedef union {
	volatile uint8_t bytes[8];
	struct {
		uint16_t temp;  //0
		uint8_t TH;  //2
		uint8_t TL;  //3
		uint8_t config;  //4
		uint8_t rrFF; //5
		uint8_t rr00; //6
		uint8_t rr10; //7
	};
} pack1_t;
volatile pack1_t pack1;



typedef union {
	#if  defined(__AVR_ATtiny25__)
	volatile uint8_t bytes[16];
	#else
	volatile uint8_t bytes[64];
	#endif
	struct {
		uint8_t status;  //1
		int16_t temp;  //2
		uint16_t voltage;  //4
		int16_t current;  //6
		uint8_t threshold; //8
		
		uint8_t page1[8]; //9
		#if  defined(__AVR_ATtiny25__)
		#else
		uint8_t page2[8]; //17
		uint8_t page3[8]; //25
		uint8_t page4[8];  //33
		uint8_t page5[8];  //41
		uint8_t page6[8];  //49
		uint8_t page7[8];  //57
		//uint8_t crc;  //65
		#endif
	};
} pack2_t;
volatile pack2_t pack2;




volatile int16_t am2302_temp;
volatile uint16_t am2302_hum;


uint8_t userRegister[1];
int16_t sRH,sT;
volatile double temperatureC,humidityRH;
volatile double l;

uint8_t max_adr=0;
#define CH0_M MAX1164x_C_SCAN0|MAX1164x_C_SGL
#define CH1_M MAX1164x_C_SCAN0|MAX1164x_C_SGL|MAX1164x_C_CS0
#define CH0_CH1 MAX1164x_C_SCAN0
//|MAX1164x_C_CS0

//T und H mit 10 multipliziert

double interp(int16_t t, int16_t h) {
double ya,yb;
uint8_t h1,h2,t1,t2;
int16_t x1,x2,y1_,y2;
	h1=0;
	h2=0;
	for(uint8_t i=0;i<5;i++) {
		int16_t hlw=pgm_read_word(&(hl[i]));
		if (h>hlw) h1=i;
	}
	h2=h1+1;
	if (h2>4) {h2=4;h1=3;}
	t2=((uint8_t)t/100); //t mit 10 multipliziert
	if (t2<1) t2=1;
	t1=t2-1;
	x1=pgm_read_word(&(hl[h1]));
	x2=pgm_read_word(&(hl[h2]));
	y1_=pgm_read_word(&(il[t1][h1]));
	y2=pgm_read_word(&(il[t1][h2]));
	double d=(double)(h-x1)/(x2-x1);
	ya=y1_+(y2-y1_)*d;
	y1_=pgm_read_word(&(il[t2][h1]));
	y2=pgm_read_word(&(il[t2][h2]));
	yb=y1_+(y2-y1_)*d;
	x1=(t1+1)*100;
	x2=x1+100;
	return exp((ya+(yb-ya)/(double)(x2-x1)*(t-x1))/1000.0);
	
}
uint8_t cmode=0;
double R0=0;
int16_t hr;
uint8_t startup=5;

int main(void){
    PRR|=(1<<PRADC);  // adc for save Power
	pack1.temp=0x0550;
	pack1.config=0x7F;
	pack1.TH=75;
	pack1.TL=70;
	pack1.rrFF=0xFF;
	pack1.rr00=0;
	pack1.rr10=0x10;
	PORTA=0xFF;
	PORTB=0xFF;
	OWINIT();
	PORTB|=(1<<PINB1);
	DDRB|=(1<<PINB1);
	PORTA|=(1<<PINA0);
	DDRA|=(1<<PINA0);
	
	MCUCR &=~(1<<PUD); //All Pins Pullup...
	MCUCR |=(1<<BODS);
	
	while(EECR & (1<<EEPE));
	EEARL=1;
	EECR |= (1<<EERE);
	hr=EEDR;
	if (hr!=0xFF) {
		hr=hr<<8;
		while(EECR & (1<<EEPE));
		EEARL=0;
		EECR |= (1<<EERE);
		hr|=EEDR;
		R0=hr/100.0;
		pack2.page3[0]=hr&0xFF;
		pack2.page3[1]=(hr>>8)&0xFF;
	}

	
	/*volatile double test=interp(0,800);
	test=interp(100,900);
	test=interp(110,400);
	test=interp(300,200);
	test=interp(300,210);
	test=interp(280,600);
	test=interp(100,900);
	*/

	WDTCSR |= ((1<<WDCE) );   // Enable the WD Change Bit//| (1<<WDE)
	WDTCSR |=   (1<<WDIE) |              // Enable WDT Interrupt
	(1<<WDP3) | (1<<WDP0);    // Set Timeout to ~8 seconds

	MCUSR=0;
	USI_TWI_Master_Initialise();
	
	SHT2x_SoftReset();
	SHT2x_ReadUserRegister(userRegister);
	//(userRegister & ~SHT2x_RES_MASK) | SHT2x_RES_10_13BIT;
	SHT2x_WriteUserRegister(userRegister); //write changed user reg
	// --- measure humidity with "Hold Master Mode (HM)" ---
	SHT2x_MeasurePoll(HUMIDITY, &sRH);
	// --- measure temperature with "Polling Mode" (no hold master) ---
	SHT2x_MeasurePoll(TEMP, &sT);
	//-- calculate humidity and temperature --
	temperatureC = SHT2x_CalcTemperatureC(sT);
	humidityRH = SHT2x_CalcRH(sRH);
	MAX1164x_config(MAX1164x_S_SEL2|MAX1164x_S_SEL0,CH0_M);//#define MAX1164x_C_CS0
	_delay_ms(30); //Internal Referenz start
	//2970 -> 1,5V	
	gcontrol=1;
	sei();
	//DDRB|=(1<<PINB1);
    while(1)   {
		if (wdcounter>0) {
			if (startup!=0) startup--;
			//PORTB|=(1<<PINB1); //Dauer 440ms
			SHT2x_MeasurePoll(HUMIDITY, &sRH);
			// --- measure temperature with "Polling Mode" (no hold master) ---
			SHT2x_MeasurePoll(TEMP, &sT);
			//-- calculate humidity and temperature --
			temperatureC = SHT2x_CalcTemperatureC(sT)*10.0;
			humidityRH = SHT2x_CalcRH(sRH)*10.0;
			double hhum=(1.0546-0.000216*temperatureC)*(humidityRH);
			am2302_hum=0.318*hhum +76.0;
			am2302_temp=temperatureC*25.6;
			//PORTB&=~(1<<PINB1);
			PORTB&=~(1<<PINB1);
			_delay_us(500);
			l=MAX1164x_read();
			PORTB|=(1<<PINB1);
			if (cmode) {
				l=l*2.048/4096+1.5; //Spannung real
			} else {
				l=l*2.048/4096; //Spannung real
			}
			if (l>1.8) {
				if (cmode==0) {
					MAX1164x_config(MAX1164x_S_SEL2|MAX1164x_S_SEL0,CH0_CH1);
					cmode=1;
					pack2.page3[4]=cmode;
				}
			}
			if (l<1.6) {
				if (cmode==1) {
					MAX1164x_config(MAX1164x_S_SEL2|MAX1164x_S_SEL0,CH0_M);
					cmode=0;
					pack2.page3[4]=cmode;
				}
				
				
			}
			hr=l*10000.0;
			pack2.page3[2]=hr&0xFF;
			pack2.page3[3]=(hr>>8)&0xFF;
			l=( 3/l- 1) *27;
			pack2.current=l*100;
			float ip=interp(temperatureC,humidityRH);
			hr=ip*1000;
			pack2.page3[5]=hr&0xFF;
			pack2.page3[6]=(hr>>8)&0xFF;
			
			l=l/ip;
			if (startup!=0){
			if (l>R0) {
				R0=l;
				hr=R0*100;
				pack2.page3[0]=hr&0xFF;
				pack2.page3[1]=(hr>>8)&0xFF;
				
				while(EECR & (1<<EEPE));
				EECR = (0<<EEPM1)|(0<<EEPM0);
				EEARL = 0;
				EEDR = hr&0xFF;
				EECR |= (1<<EEMPE);
				EECR |= (1<<EEPE);
				while(EECR & (1<<EEPE));
				EECR = (0<<EEPM1)|(0<<EEPM0);
				EEARL = 1;
				EEDR = hr>>8;
				EECR |= (1<<EEMPE);
				EECR |= (1<<EEPE);

			}
			}
			 l=l/R0;
			l=log(1/l);
			l=l*160; //fuer DS18B20
			wdcounter=0;
			
		}
	
		if (gcontrol==1) {
			uint16_t w=l;
			uint8_t t8=w>>4;
			uint8_t af=0;
			if (t8>pack1.TH) af=1;
			if (t8<=pack1.TL) af=1; 
			cli();
			pack1.temp=w;
			//pack.temp++;
			alarmflag=af;
			sei();			
			EXTERN_SLEEP();		
		}
		if ((gcontrol==2)||(gcontrol==3)) {
			gcontrol=0;
			
		}

		
#if  defined(__AVR_ATtiny25__)||defined(__AVR_ATtiny45__)  || defined(__AVR_ATtiny85__)
			if (((TIMSK & (1<<TOIE0))==0)&& (mode==0))
#endif			
#if  defined(__AVR_ATtiny24__)||defined(__AVR_ATtiny44__)  || defined(__AVR_ATtiny84__) ||defined(__AVR_ATtiny24A__)||defined(__AVR_ATtiny44A__)  || defined(__AVR_ATtiny84A__)
			if (((TIMSK0 & (1<<TOIE0))==0)&& (mode==0))
#endif
			  {

			MCUCR|=(1<<SE)|(1<<SM1);
			MCUCR&=~(1<<ISC01);
		} else {
			MCUCR|=(1<<SE);
			MCUCR&=~(1<<SM1);
		}
		asm("SLEEP");
   }


}